@@ -50,6 +50,40 @@ class DownloaderArt:
         self.__in_albums_download = False
         self.__in_artists_download = False
 
+    def add_artist_artwork_from_uri(self, artist, uri,
+                                    cancellable, storage_type):
+        """
+            Add artist artwork
+            @param artist as str
+            @param uri as str
+            @param cancellable as Gio.Cancellable
+            @param storage_type as int
+        """
+        def on_uri_content(uri, status, data):
+            if status:
+                App().art.add_artist_artwork(artist, data, storage_type)
+                emit_signal(self, "artist-artwork-changed", artist)
+        if uri:
+            App().task_helper.load_uri_content(uri,
+                                               cancellable,
+                                               on_uri_content)
+
+    def add_album_artwork_from_uri(self, album, uri, cancellable):
+        """
+            Add album artwork
+            @param album as Album
+            @param uri as str
+            @param cancellable as Gio.Cancellable
+        """
+        def on_uri_content(uri, status, data):
+            if status:
+                App().art.add_album_artwork(album, data)
+                emit_signal(self, "album-artwork-changed", album.id)
+        if uri:
+            App().task_helper.load_uri_content(uri,
+                                               cancellable,
+                                               on_uri_content)
+
     def search_album_artworks(self, artist, album, cancellable):
         """
             Search album artworks
@@ -60,8 +94,12 @@ class DownloaderArt:
         """
         results = []
         for api in self.__album_methods.keys():
+            if cancellable.is_cancelled():
+                return
             uris = self.__album_methods[api](artist, album, cancellable)
             for uri in uris:
+                if cancellable.is_cancelled():
+                    return
                 results.append((uri, api))
         emit_signal(self, "uri-artwork-found", results)
 
@@ -74,8 +112,12 @@ class DownloaderArt:
         """
         results = []
         for api in self.__artist_methods.keys():
+            if cancellable.is_cancelled():
+                return
             uris = self.__artist_methods[api](artist, cancellable)
             for uri in uris:
+                if cancellable.is_cancelled():
+                    return
                 results.append((uri, api))
         emit_signal(self, "uri-artwork-found", results)
 
@@ -145,11 +187,11 @@ class DownloaderArt:
             Get artist artwork using AutdioDB
             @param artist as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
-            @thread safe
+            @return uris as [str]
         """
         if not get_network_available("AUDIODB"):
             return []
+        uris = []
         try:
             artist = GLib.uri_escape_string(artist, None, True)
             uri = "https://theaudiodb.com/api/v1/json/"
@@ -158,53 +200,50 @@ class DownloaderArt:
                 uri, cancellable)
             if status:
                 decode = json.loads(data.decode("utf-8"))
-                uri = None
                 for item in decode["artists"]:
                     for key in ["strArtistFanart", "strArtistThumb"]:
                         uri = item[key]
                         if uri is not None:
-                            return [uri]
+                            uris.append(uri)
         except Exception as e:
-            Logger.warning("%s %s", e, artist)
-            Logger.warning(
-                "DownloaderArt::_get_audiodb_artist_artwork_uri: %s", data)
-        return []
+            Logger.warning("AudioDB: %s %s", e, artist)
+            uris = []
+        return uris
 
     def _get_deezer_artist_artwork_uri(self, artist, cancellable=None):
         """
             Get artist artwork using Deezer
             @param artist as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
-            @tread safe
+            @return uris as [str]
         """
         if not get_network_available("DEEZER"):
             return []
+        uris = []
         try:
             artist_formated = GLib.uri_escape_string(
                 artist, None, True).replace(" ", "+")
             uri = "https://api.deezer.com/search/artist/?" +\
-                  "q=%s&output=json&index=0&limit=1" % artist_formated
+                  "q=%s&output=json&index=0" % artist_formated
             (status, data) = App().task_helper.load_uri_content_sync(
                 uri, cancellable)
             if status:
-                uri = None
+                artist = noaccents(artist.lower())
                 decode = json.loads(data.decode("utf-8"))
-                uri = decode["data"][0]["picture_xl"]
-                return [uri]
+                for item in decode["data"]:
+                    uri = item["picture_xl"]
+                    uris.append(uri)
         except Exception as e:
-            Logger.warning("%s %s", e, artist)
-            Logger.warning(
-                "DownloaderArt::_get_deezer_artist_artwork_uri(): %s", data)
-        return []
+            Logger.warning("Deezer: %s %s", e, artist)
+            uris = []
+        return uris
 
     def _get_fanarttv_artist_artwork_uri(self, artist, cancellable=None):
         """
             Get artist artwork using FanartTV
             @param artist as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
-            @thread safe
+            @return uris as [str]
         """
         if not get_network_available("FANARTTV"):
             return []
@@ -221,9 +260,8 @@ class DownloaderArt:
                 for item in decode["artistbackground"]:
                     uris.append(item["url"])
         except Exception as e:
-            Logger.warning("%s %s", e, artist)
-            Logger.warning(
-                "DownloaderArt::_get_fanarttv_artist_artwork_uri: %s", data)
+            Logger.warning("FanartTV: %s %s", e, artist)
+            uris = []
         return uris
 
     def _get_spotify_artist_artwork_uri(self, artist, cancellable=None):
@@ -231,11 +269,11 @@ class DownloaderArt:
             Get artist artwork using Spotify
             @param artist as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
-            @tread safe
+            @return uris as [str]
         """
         if not get_network_available("SPOTIFY"):
             return []
+        uris = []
         try:
             artist_formated = GLib.uri_escape_string(
                 artist, None, True).replace(" ", "+")
@@ -249,18 +287,15 @@ class DownloaderArt:
              data) = App().task_helper.load_uri_content_sync_with_headers(
                     uri, headers, cancellable)
             if status:
-                uri = None
+                artist = noaccents(artist.lower())
                 decode = json.loads(data.decode("utf-8"))
                 for item in decode["artists"]["items"]:
-                    if noaccents(item["name"].lower()) ==\
-                            noaccents(artist.lower()):
-                        uri = item["images"][0]["url"]
-                        return [uri]
+                    uri = item["images"][0]["url"]
+                    uris.append(uri)
         except Exception as e:
-            Logger.warning("%s %s", e, artist)
-            Logger.warning(
-                "DownloaderArt::_get_spotify_artist_artwork_uri(): %s", data)
-        return []
+            Logger.warning("Spotify: %s %s", e, artist)
+            uris = []
+        return uris
 
     def _get_deezer_album_artwork_uri(self, artist, album, cancellable=None):
         """
@@ -268,11 +303,11 @@ class DownloaderArt:
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
-            @tread safe
+            @return uris as [str]
         """
         if not get_network_available("DEEZER"):
             return []
+        uris = []
         try:
             album_formated = GLib.uri_escape_string(album, None, True)
             uri = "https://api.deezer.com/search/album/?" +\
@@ -280,18 +315,18 @@ class DownloaderArt:
             (status, data) = App().task_helper.load_uri_content_sync(
                 uri, cancellable)
             if status:
+                artist = noaccents(artist.lower())
                 decode = json.loads(data.decode("utf-8"))
-                uri = None
                 for item in decode["data"]:
-                    if noaccents(item["artist"]["name"].lower()) ==\
-                            noaccents(artist.lower()):
-                        uri = item["cover_xl"]
-                        return [uri]
+                    item_artist = noaccents(item["artist"]["name"].lower())
+                    if artist and artist != item_artist:
+                        continue
+                    uri = item["cover_xl"]
+                    uris.append(uri)
         except Exception as e:
-            Logger.warning("%s %s %s", e, artist, album)
-            Logger.warning("DownloaderArt::__get_deezer_album_artwork_uri: %s",
-                           data)
-        return []
+            Logger.warning("Deezer: %s %s %s", e, artist, album)
+            uris = []
+        return uris
 
     def _get_fanarttv_album_artwork_uri(self, artist, album, cancellable=None):
         """
@@ -299,7 +334,7 @@ class DownloaderArt:
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
+            @return uris as [str]
             @thread safe
         """
         if not get_network_available("FANARTTV"):
@@ -318,9 +353,8 @@ class DownloaderArt:
                 for cover in decode["albums"][mbid]["albumcover"]:
                     uris.append(cover["url"])
         except Exception as e:
-            Logger.warning("%s %s %s", e, artist, album)
-            Logger.warning(
-                "DownloaderArt::_get_fanarttv_album_artwork_uri: %s", data)
+            Logger.warning("FanartTV: %s %s %s", e, artist, album)
+            uris = []
         return uris
 
     def _get_spotify_album_artwork_uri(self, artist, album, cancellable=None):
@@ -329,12 +363,13 @@ class DownloaderArt:
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
+            @return uris as [str]
             @tread safe
         """
-        if not get_network_available("SPOTIFY"):
+        # Spotify API need an artist
+        if not get_network_available("SPOTIFY") or not artist:
             return []
-        artists_spotify_ids = []
+        uris = []
         try:
             artist_formated = GLib.uri_escape_string(
                 artist, None, True).replace(" ", "+")
@@ -348,28 +383,31 @@ class DownloaderArt:
              data) = App().task_helper.load_uri_content_sync_with_headers(
                     uri, headers, cancellable)
             if status:
+                artist = noaccents(artist.lower())
                 decode = json.loads(data.decode("utf-8"))
+                artist_spotify_id = None
                 for item in decode["artists"]["items"]:
-                    artists_spotify_ids.append(item["id"])
-
-            for artist_spotify_id in artists_spotify_ids:
-                uri = "https://api.spotify.com/v1/artists/" +\
-                      "%s/albums" % artist_spotify_id
-                (status,
-                 data) = App().task_helper.load_uri_content_sync_with_headers(
-                    uri, headers, cancellable)
-                if status:
-                    decode = json.loads(data.decode("utf-8"))
-                    uri = None
-                    for item in decode["items"]:
-                        if noaccents(item["name"].lower()) ==\
-                                noaccents(album.lower()):
-                            return [item["images"][0]["url"]]
+                    item_artist = noaccents(item["name"].lower())
+                    if artist == item_artist:
+                        artist_spotify_id = item["id"]
+                        break
+            if artist_spotify_id is None:
+                return []
+            uri = "https://api.spotify.com/v1/artists/" +\
+                  "%s/albums" % artist_spotify_id
+            (status,
+             data) = App().task_helper.load_uri_content_sync_with_headers(
+                uri, headers, cancellable)
+            if status:
+                decode = json.loads(data.decode("utf-8"))
+                for item in decode["items"]:
+                    if noaccents(item["name"].lower()) ==\
+                            noaccents(album.lower()):
+                        uris.append(item["images"][0]["url"])
         except Exception as e:
-            Logger.warning("%s %s %s", e, artist, album)
-            Logger.warning(
-                "DownloaderArt::_get_album_art_spotify_uri: %s", data)
-        return []
+            Logger.warning("Spotify: %s %s %s", e, artist, album)
+            uris = []
+        return uris
 
     def _get_itunes_album_artwork_uri(self, artist, album, cancellable=None):
         """
@@ -377,11 +415,12 @@ class DownloaderArt:
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
+            @return uris as [str]
             @tread safe
         """
         if not get_network_available("ITUNES"):
             return []
+        uris = []
         try:
             album_formated = GLib.uri_escape_string(
                 album, None, True).replace(" ", "+")
@@ -390,18 +429,18 @@ class DownloaderArt:
             (status, data) = App().task_helper.load_uri_content_sync(
                 uri, cancellable)
             if status:
+                artist = noaccents(artist.lower())
                 decode = json.loads(data.decode("utf-8"))
                 for item in decode["results"]:
-                    if noaccents(item["artistName"].lower()) ==\
-                            noaccents(artist.lower()):
-                        uri = item["artworkUrl60"].replace("60x60",
-                                                           "1024x1024")
-                        return [uri]
+                    item_artist = noaccents(item["artistName"].lower())
+                    if artist and artist != item_artist:
+                        continue
+                    uris.append(item["artworkUrl60"].replace(
+                        "60x60", "1024x1024"))
         except Exception as e:
-            Logger.warning("%s %s %s", e, artist, album)
-            Logger.warning(
-                "DownloaderArt::_get_album_art_itunes_uri: %s", data)
-        return []
+            Logger.warning("Itunes: %s %s %s", e, artist, album)
+            uris = []
+        return uris
 
     def _get_audiodb_album_artwork_uri(self, artist, album, cancellable=None):
         """
@@ -409,11 +448,12 @@ class DownloaderArt:
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return uri as str
+            @return uris as [str]
             @thread safe
         """
         if not get_network_available("AUDIODB"):
             return []
+        uris = []
         try:
             album = GLib.uri_escape_string(album, None, True)
             artist = GLib.uri_escape_string(artist, None, True)
@@ -426,14 +466,16 @@ class DownloaderArt:
             if status:
                 decode = json.loads(data.decode("utf-8"))
                 if decode["album"]:
+                    artist = noaccents(artist.lower())
                     for item in decode["album"]:
-                        uri = item["strAlbumThumb"]
-                        return [uri]
+                        item_artist = noaccents(item["strArtist"].lower())
+                        if artist and artist != item_artist:
+                            continue
+                        uris.append(item["strAlbumThumb"])
         except Exception as e:
-            Logger.warning("%s %s %s", e, artist, album)
-            Logger.warning(
-                "DownloaderArt::_get_audiodb_album_artwork_uri: %s", data)
-        return []
+            Logger.warning("AudioDB: %s %s %s", e, artist, album)
+            uris = []
+        return uris
 
     def _get_lastfm_album_artwork_uri(self, artist, album, cancellable=None):
         """
@@ -441,7 +483,7 @@ class DownloaderArt:
             @param artist as str
             @param album as str
             @param cancellable as Gio.Cancellable
-            @return [str]
+            @return uris as [str]
             @tread safe
         """
         if not get_network_available("LASTFM"):
@@ -453,9 +495,7 @@ class DownloaderArt:
             artwork_uri = payload["image"][-1]["#text"]
             return [artwork_uri]
         except Exception as e:
-            Logger.warning("%s %s %s", e, artist, album)
-            Logger.warning(
-                "DownloaderArt::_get_album_art_lastfm_uri: %s", payload)
+            Logger.warning("Last.fm: %s %s %s", e, artist, album)
         return []
 
 #######################
@@ -511,14 +551,10 @@ class DownloaderArt:
                 for api in self.__artist_methods.keys():
                     result = self.__artist_methods[api](artist)
                     for uri in result:
-                        (status,
-                         data) = App().task_helper.load_uri_content_sync(
-                            uri, None)
-                        if status:
-                            found = True
-                            App().art.add_artist_artwork(
-                                artist, data, StorageType.COLLECTION)
-                            break
+                        found = True
+                        self.add_artist_artwork_from_uri(
+                            artist, uri, None, StorageType.COLLECTION)
+                        break
                     # Found, do not search in another helper
                     if found:
                         break
@@ -551,19 +587,15 @@ class DownloaderArt:
                 for api in self.__album_methods.keys():
                     result = self.__album_methods[api](artist, album)
                     for uri in result:
-                        (status,
-                         data) = App().task_helper.load_uri_content_sync(uri,
-                                                                         None)
-                        if status:
-                            found = True
-                            App().art.save_album_artwork(Album(album_id), data)
-                            break
+                        self.add_album_artwork_from_uri(
+                            Album(album_id), uri, None)
+                        break
                     # Found, do not search in another helper
                     if found:
                         break
                 # Not found, save empty artwork
                 if not found:
-                    App().art.save_album_artwork(Album(album_id), None)
+                    App().art.add_album_artwork(Album(album_id), None)
         except Exception as e:
             Logger.error("DownloaderArt::__cache_albums_artwork: %s" % e)
         self.__in_albums_download = False
