@@ -38,9 +38,10 @@ class Playlists(GObject.GObject):
     __LOCAL_PATH = GLib.get_user_data_dir() + "/lollypop"
     _DB_PATH = "%s/playlists.db" % __LOCAL_PATH
     __gsignals__ = {
-        # Add or remove a playlist
-        "playlists-changed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
-        # Objects added/removed to/from playlist
+        "playlists-added": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
+        "playlists-removed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
+        "playlists-updated": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
+        "playlists-renamed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         "playlist-track-added": (
             GObject.SignalFlags.RUN_FIRST, None, (int, str)),
         "playlist-track-removed": (
@@ -111,7 +112,7 @@ class Playlists(GObject.GObject):
                                   VALUES (?, ?)",
                                  (name, 0))
             lastrowid = result.lastrowid
-        emit_signal(self, "playlists-changed", lastrowid)
+        emit_signal(self, "playlists-added", lastrowid)
         return lastrowid
 
     def exists(self, playlist_id):
@@ -141,7 +142,7 @@ class Playlists(GObject.GObject):
                         SET name=?\
                         WHERE rowid=?",
                         (name, playlist_id))
-        emit_signal(self, "playlists-changed", playlist_id)
+        emit_signal(self, "playlists-renamed", playlist_id)
         App().art.remove_artwork_from_cache("playlist_" + name, "ROUNDED")
 
     def remove(self, playlist_id):
@@ -157,7 +158,7 @@ class Playlists(GObject.GObject):
             sql.execute("DELETE FROM tracks\
                         WHERE playlist_id=?",
                         (playlist_id,))
-        emit_signal(self, "playlists-changed", playlist_id)
+        emit_signal(self, "playlists-removed", playlist_id)
         App().art.remove_artwork_from_cache("playlist_" + name, "ROUNDED")
 
     def clear(self, playlist_id):
@@ -179,10 +180,10 @@ class Playlists(GObject.GObject):
         """
         if self.exists_track(playlist_id, uri):
             return
-        if signal:
-            emit_signal(self, "playlist-track-added", playlist_id, uri)
         with SqlCursor(self, True) as sql:
             sql.execute("INSERT INTO tracks VALUES (?, ?)", (playlist_id, uri))
+        if signal:
+            emit_signal(self, "playlist-track-added", playlist_id, uri)
 
     def add_uris(self, playlist_id, uris, signal=False):
         """
@@ -215,11 +216,11 @@ class Playlists(GObject.GObject):
         """
         if not self.exists_track(playlist_id, uri):
             return
-        if signal:
-            emit_signal(self, "playlist-track-removed", playlist_id, uri)
         with SqlCursor(self, True) as sql:
             sql.execute("DELETE FROM tracks WHERE uri=? AND playlist_id=?",
                         (uri, playlist_id))
+        if signal:
+            emit_signal(self, "playlist-track-removed", playlist_id, uri)
 
     def remove_uris(self, playlist_id, uris, signal=False):
         """
@@ -269,7 +270,7 @@ class Playlists(GObject.GObject):
 
     def get_track_uris(self, playlist_id):
         """
-            Return availables track uris for playlist
+            Return available track uris for playlist
             @param playlist_id as int
             @return [str]
         """
@@ -279,6 +280,21 @@ class Playlists(GObject.GObject):
                                   WHERE playlist_id=?", (playlist_id,))
             return list(itertools.chain(*result))
 
+    def get_smart_track_uris(self, playlist_id):
+        """
+            Return available track uris for playlist
+            @param playlist_id as int
+            @return [str]
+        """
+        request = self.get_smart_sql(playlist_id)
+        # We need to inject skipped/storage_type
+        storage_type = get_default_storage_type()
+        split = request.split("ORDER BY")
+        split[0] += " AND tracks.loved != %s" % Type.NONE
+        split[0] += " AND tracks.storage_type&%s " % storage_type
+        track_ids = App().db.execute("ORDER BY".join(split))
+        return [Track(track_id).uri for track_id in track_ids]
+
     def get_track_ids(self, playlist_id):
         """
             Return availables track ids for playlist
@@ -289,20 +305,23 @@ class Playlists(GObject.GObject):
         limit = App().settings.get_value("view-limit").get_int32()
         storage_type = get_default_storage_type()
         if playlist_id == Type.POPULARS:
-            track_ids = App().tracks.get_rated(storage_type, limit)
-            for track in App().tracks.get_populars(storage_type, limit):
-                track_ids.append(track)
+            track_ids = App().tracks.get_populars([], storage_type,
+                                                  False, limit)
         elif playlist_id == Type.RECENTS:
             track_ids = App().tracks.get_recently_listened_to(storage_type,
+                                                              False,
                                                               limit)
         elif playlist_id == Type.LITTLE:
-            track_ids = App().tracks.get_little_played(storage_type, limit)
+            track_ids = App().tracks.get_little_played(storage_type,
+                                                       False,
+                                                       limit)
         elif playlist_id == Type.RANDOMS:
-            track_ids = App().tracks.get_randoms(storage_type, limit)
+            track_ids = App().tracks.get_randoms([], storage_type,
+                                                 False, limit)
         elif playlist_id == Type.ALL:
-            track_ids = App().tracks.get_ids(storage_type)
+            track_ids = App().tracks.get_ids(storage_type, False)
         elif playlist_id == Type.LOVED:
-            track_ids = App().tracks.get_loved_track_ids(storage_type)
+            track_ids = App().tracks.get_loved_track_ids([], storage_type)
         else:
             with SqlCursor(self) as sql:
                 result = sql.execute("SELECT music.tracks.rowid\
@@ -497,7 +516,7 @@ class Playlists(GObject.GObject):
                         SET smart_enabled=?\
                         WHERE rowid=?",
                         (smart, playlist_id))
-            emit_signal(self, "playlists-changed", playlist_id)
+            emit_signal(self, "playlists-updated", playlist_id)
 
     def set_smart_sql(self, playlist_id, request):
         """
@@ -513,7 +532,7 @@ class Playlists(GObject.GObject):
                         SET smart_sql=?\
                         WHERE rowid=?",
                         (request, playlist_id))
-            emit_signal(self, "playlists-changed", playlist_id)
+            emit_signal(self, "playlists-updated", playlist_id)
 
     def get_position(self, playlist_id, track_id):
         """
@@ -582,20 +601,30 @@ class Playlists(GObject.GObject):
             name = self.get_name(playlist_id)
             # Clear cache
             App().art.remove_artwork_from_cache("playlist_" + name, "ROUNDED")
-            uri = self.get_sync_uri(playlist_id)
-            if uri is None:
+            playlist_uri = self.get_sync_uri(playlist_id)
+            if playlist_uri is None:
                 return
-            f = Gio.File.new_for_uri(uri)
+            f = Gio.File.new_for_uri(playlist_uri)
             if not f.query_exists() and not create:
                 return
-            uris = self.get_track_uris(playlist_id)
+            if self.get_smart(playlist_id):
+                uris = self.get_smart_track_uris(playlist_id)
+            else:
+                uris = self.get_track_uris(playlist_id)
             if not uris:
                 return
             stream = f.replace(None, False,
                                Gio.FileCreateFlags.REPLACE_DESTINATION, None)
             stream.write("#EXTM3U\n".encode("utf-8"))
+            playlist_dir_uri = playlist_uri.replace(f.get_basename(), "")
             for uri in uris:
-                string = "%s\n" % uri
+                if uri.startswith("web://"):
+                    continue
+                if playlist_dir_uri in uri:
+                    filepath = uri.replace(playlist_dir_uri, "")
+                    string = "%s\n" % GLib.uri_unescape_string(filepath, None)
+                else:
+                    string = "%s\n" % uri
                 stream.write(string.encode("utf-8"))
             stream.close()
         except Exception as e:
@@ -622,17 +651,19 @@ class Playlists(GObject.GObject):
             sql.execute("UPDATE playlists SET synced = synced & ~(1<<?)",
                         (index,))
 
-    def import_tracks(self, f):
+    def import_tracks(self, uri):
         """
             Import file as playlist
-            @param f as Gio.File
+            @param uri as str
         """
+        f = Gio.File.new_for_uri(uri)
         # Create playlist and get id
         basename = ".".join(f.get_basename().split(".")[:-1])
         playlist_id = self.get_id(basename)
-        if playlist_id is None:
-            playlist_id = self.add(basename)
-
+        # Do not reimport playlists
+        if playlist_id is not None:
+            return
+        playlist_id = self.add(basename)
         # Check mtime has been updated
         with SqlCursor(self) as sql:
             result = sql.execute("SELECT mtime\
